/*
 * Copyright 2022 Esri
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.app

import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity

import android.graphics.Color
import com.esri.arcgisruntime.ArcGISRuntimeEnvironment
import com.esri.arcgisruntime.arcgisservices.LabelDefinition
import com.esri.arcgisruntime.data.ServiceFeatureTable
import com.esri.arcgisruntime.layers.FeatureLayer
import com.esri.arcgisruntime.mapping.ArcGISMap
import com.esri.arcgisruntime.mapping.BasemapStyle
import com.esri.arcgisruntime.mapping.Viewpoint
import com.esri.arcgisruntime.mapping.labeling.ArcadeLabelExpression
import com.esri.arcgisruntime.mapping.view.MapView
import com.esri.arcgisruntime.symbology.ClassBreaksRenderer
import com.esri.arcgisruntime.symbology.ClassBreaksRenderer.ClassBreak
import com.esri.arcgisruntime.symbology.PictureMarkerSymbol
import com.esri.arcgisruntime.symbology.SimpleFillSymbol
import com.esri.arcgisruntime.symbology.SimpleLineSymbol
import com.esri.arcgisruntime.symbology.SimpleRenderer
import com.esri.arcgisruntime.symbology.TextSymbol
import com.esri.arcgisruntime.symbology.UniqueValueRenderer
import com.esri.arcgisruntime.symbology.UniqueValueRenderer.UniqueValue

import com.example.app.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity() {

    private val activityMainBinding by lazy {
        ActivityMainBinding.inflate(layoutInflater)
    }

    private val mapView: MapView by lazy {
        activityMainBinding.mapView
    }

    private lateinit var map: ArcGISMap

    private val parksAndOpenSpaces =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Parks_and_Open_Space/FeatureServer/0"
    private val trails =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0"
    private val trailheads =
        "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trailheads/FeatureServer/0"
    private val trailheadImage =
        "https://static.arcgis.com/images/Symbols/NPS/npsPictograph_0231b.png"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(activityMainBinding.root)

        setApiKeyForApp()

        setupMap()
    }
    override fun onPause() {
        mapView.pause()
        super.onPause()
    }

    override fun onResume() {
        super.onResume()
        mapView.resume()
    }

    override fun onDestroy() {
        mapView.dispose()
        super.onDestroy()
    }

    private fun setApiKeyForApp() {
        // set your API key
        // Note: it is not best practice to store API keys in source code. The API key is referenced
        // here for the convenience of this tutorial.

        ArcGISRuntimeEnvironment.setApiKey("YOUR_API_KEY")
    }

    // set up your map here. You will call this method from onCreate()
    private fun setupMap() {
        // create a map with the BasemapStyle streets
        map = ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC)

        // set the map to be displayed in the layout's MapView
        mapView.map = map

        // set the viewpoint, Viewpoint(latitude, longitude, scale)
        mapView.setViewpoint(Viewpoint(34.0270, -118.8050, 72000.0))

        addOpenSpaceLayer()

        addTrailsLayer()

        addBikeOnlyTrailsLayer()

        addNoBikeTrailsLayer()

        addTrailheadsLayer()

    }

    private fun addFeatureLayer(featureServiceUri: String): FeatureLayer {
        // Create a service feature table from a Uri.
        val serviceFeatureTable = ServiceFeatureTable(featureServiceUri)
        // Create a feature layer from the service feature table.
        val featureLayer = FeatureLayer(serviceFeatureTable)
        // Add the feature layer to the map's list of operational layers.
        map.operationalLayers.add(featureLayer)
        // return the feature layer
        return featureLayer
    }

    private fun addOpenSpaceLayer() {
        // Create a parks and open spaces feature layer and add it to the map view.
        val featureLayer = addFeatureLayer(parksAndOpenSpaces)

        // Create fill symbols.
        val magentaFillSymbol = SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.MAGENTA, null)
        val greenFillSymbol = SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.GREEN, null)
        val blueFillSymbol = SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.BLUE, null)
        val redFillSymbol = SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, Color.RED, null)

        // Create a unique value for natural areas, regional open spaces, local parks, and regional recreation parks.
        val naturalAreas = UniqueValue(
            "Natural Areas",
            "Natural Areas",
            magentaFillSymbol,
            listOf("Natural Areas")
        )

        val regionalOpenSpace = UniqueValue(
            "Regional Open Space",
            "Regional Open Space",
            greenFillSymbol,
            listOf("Regional Open Space")
        )

        val localPark = UniqueValue(
            "Local Park",
            "Local Park",
            blueFillSymbol,
            listOf("Local Park")
        )

        val regionalRecreationPark = UniqueValue(
            "Regional Recreation Park",
            "Regional Recreation Park",
            redFillSymbol,
            listOf("Regional Recreation Park")
        )

        val uniqueValuesList =
            listOf(naturalAreas, regionalOpenSpace, localPark, regionalRecreationPark)

        // Create and assign a unique value renderer to the feature layer.
        val openSpacesUniqueValueRenderer =
            UniqueValueRenderer(listOf("TYPE"), uniqueValuesList, "Open Spaces", null)
        featureLayer.renderer = openSpacesUniqueValueRenderer

        // Set the layer opacity to semi-transparent.
        featureLayer.opacity = 0.2f
    }

    private fun addTrailsLayer() {
        // Create a trails feature layer and add it to the map view.
        val featureLayer = addFeatureLayer(trails)

        // Create simple line symbols.
        val firstClassSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.MAGENTA, 3.0f)
        val secondClassSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.MAGENTA, 4.0f)
        val thirdClassSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.MAGENTA, 5.0f)
        val fourthClassSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.MAGENTA, 6.0f)
        val fifthClassSymbol = SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, Color.MAGENTA, 7.0f)

        // Create 5 class breaks.
        val firstClassBreak =
            ClassBreak("Under 500", "0 - 500", 0.0, 500.0, firstClassSymbol)
        val secondClassBreak =
            ClassBreak("501 to 1000", "501 - 1000", 501.0, 1000.0, secondClassSymbol)
        val thirdClassBreak =
            ClassBreak("1001 to 1500", "1001 - 1500", 1001.0, 1500.0, thirdClassSymbol)
        val fourthClassBreak =
            ClassBreak("1501 to 2000", "1501 - 2000", 1501.0, 2000.0, fourthClassSymbol)
        val fifthClassBreak =
            ClassBreak("2001 to 2300", "2001 - 2300", 2001.0, 2300.0, fifthClassSymbol)

        val elevationBreaks = listOf(
            firstClassBreak,
            secondClassBreak,
            thirdClassBreak,
            fourthClassBreak,
            fifthClassBreak
        )

        // Create and assign a class breaks renderer to the feature layer.
        val elevationClassBreaksRenderer = ClassBreaksRenderer("ELEV_GAIN", elevationBreaks)
        featureLayer.renderer = elevationClassBreaksRenderer

        // Set the layer opacity to semi-transparent.
        featureLayer.opacity = 0.75f
    }

    private fun addBikeOnlyTrailsLayer() {
        // Create a trails feature layer and add it to the map view.
        val featureLayer = addFeatureLayer(trails)

        // Write a definition expression to filter for trails that permit the use of bikes.
        featureLayer.definitionExpression = "USE_BIKE = 'Yes'"

        // Create and assign a simple renderer to the feature layer.
        val bikeTrailSymbol =
            SimpleLineSymbol(SimpleLineSymbol.Style.DOT, Color.BLUE, 2.0f)
        val bikeTrailRenderer = SimpleRenderer(bikeTrailSymbol)
        featureLayer.renderer = bikeTrailRenderer
    }

    private fun addNoBikeTrailsLayer() {
        // Create a trails feature layer and add it to the map view.
        val featureLayer: FeatureLayer = addFeatureLayer(trails)

        // Write a definition expression to filter for trails that don't permit the use of bikes.
        featureLayer.definitionExpression = "USE_BIKE = 'No'"

        // Create and assign a simple renderer to the feature layer.
        val noBikeTrailSymbol =
            SimpleLineSymbol(SimpleLineSymbol.Style.DOT, Color.YELLOW, 2.0f)
        val noBikeTrailRenderer = SimpleRenderer(noBikeTrailSymbol)
        featureLayer.renderer = noBikeTrailRenderer
    }

    private fun makeLabelDefinition(labelAttribute: String): LabelDefinition {
        // Create a new text symbol.
        val labelTextSymbol = TextSymbol().apply {
            color = Color.WHITE
            size = 12.0f
            haloColor = Color.RED
            haloWidth = 1.0f
            fontFamily = "Arial"
            fontStyle = TextSymbol.FontStyle.ITALIC
            fontWeight = TextSymbol.FontWeight.NORMAL
        }

        // Create a new Arcade label expressions based on the field name.
        val labelExpression = ArcadeLabelExpression("\$feature." + labelAttribute)

        // Create and return the label definition.
        return LabelDefinition(labelExpression, labelTextSymbol)
    }

    private fun addTrailheadsLayer() {
        // Create a trail heads feature layer and add it to the map view.
        val featureLayer: FeatureLayer = addFeatureLayer(trailheads)

        // Create a new picture marker symbol that uses the trailhead image.
        val pictureMarkerSymbol = PictureMarkerSymbol(trailheadImage).apply {
            height = 18.0f
            width = 18.0f
        }

        // Create a new simple renderer based on the picture marker symbol.
        val simpleRenderer = SimpleRenderer(pictureMarkerSymbol)

        // Set the feature layer's renderer and enable labels.
        featureLayer.renderer = simpleRenderer
        featureLayer.isLabelsEnabled = true

        // Create the label definition.
        val trailHeadsDefinition = makeLabelDefinition("TRL_NAME")

        // Add the label definition to the layer's label definition collection.
        featureLayer.labelDefinitions.add(trailHeadsDefinition)
    }

}

